
# q-expansion of newform 4882.2.a.c, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4882
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [0, 2], [3, 0], [-1, -2], [1, -2], [-6, 1], [-1, -4], [-4, -2], [-1, 0], [7, 1], [-7, 0], [4, -8], [7, -1], [-4, -2], [7, 0], [-7, 6], [2, -7], [-3, 5], [-7, 1], [3, 3], [-3, 11], [-7, -5], [-8, 3], [0, 1], [-2, 5], [9, -8], [-3, -6], [7, 4], [-4, 5], [-5, -3], [-3, -5], [-9, 13], [13, -3], [-7, 4], [1, -1], [-1, 7], [-7, 1], [-19, -4], [-4, 16], [5, -8], [1, 1], [-6, 2], [18, -4], [-13, 6], [-3, -12], [-14, 3], [-9, -7], [-10, -2], [9, -2], [5, -20], [-16, 19], [-19, 3], [17, -18], [0, 13], [6, 5], [-6, 4], [1, 5], [1, -8], [-5, 7], [17, -5], [-12, 21], [17, -2], [-7, -3], [16, 0], [11, -4], [-19, 10], [11, 8], [1, -5], [-10, 5], [14, 12], [5, 0], [-2, -7], [-17, 20], [-4, 23], [5, 4], [-5, -12], [-15, -2], [-5, -6], [-6, 25], [11, -19], [5, 10], [32, -3], [-9, 18], [10, -17], [-23, 2], [-3, 11], [0, -14], [-22, 11], [-25, 4], [-12, -1], [16, -12], [-25, -6], [-9, 0], [13, -4], [-5, 8], [-18, -2], [3, -24], [8, 3], [-4, 0], [-3, 22], [-7, -14], [-23, 2], [-18, 26], [26, -2], [21, 10], [-11, -11], [-26, -2], [9, -5], [-30, 6], [24, -12], [-25, -3], [0, -26], [8, -31], [23, -2], [-31, 28], [-14, -10], [-29, 16], [44, -2], [-3, -10], [0, 3], [-18, -13], [-2, -16], [31, 8], [-12, -6], [-9, 11], [12, 15], [24, -18], [6, -2], [17, -23], [-32, 3], [25, -19], [17, -42], [5, -17], [-8, -6], [0, -4], [20, -9], [11, -26], [-15, 30], [29, -11], [-15, 18], [31, -29], [-19, -1], [10, -16], [-9, -9], [27, -1], [-21, 7], [7, -21], [-19, -8], [-29, 30], [15, -7], [-14, 26], [-3, -1], [12, 9], [0, 13], [34, 7], [-4, -1], [25, -27], [45, -5], [-7, 3], [39, 7], [-43, 18], [-41, 0], [11, 6], [42, 10], [-12, 25], [34, -11], [-1, 18], [5, -20], [-2, -6], [-23, -10], [-8, -21], [23, -1], [16, -35], [37, 1], [5, -12], [24, -5], [-12, -1], [14, -9], [-24, -12], [2, 24], [-24, 1], [23, -42], [6, -2], [25, -25], [12, 10], [-28, -16], [-33, 16], [-12, -32], [-22, 21], [-52, 14], [22, -5], [-31, 8], [4, 22], [18, 2], [33, -18], [22, 22], [25, -18], [-9, 27], [21, -25], [-34, 25], [-3, 1], [-7, -20], [-30, 18], [8, -19], [-50, 20], [-34, -2], [11, 4], [-33, 39], [-24, 25], [52, -24], [-8, 47], [-28, 15], [-48, -11], [0, -34], [-22, 40], [-21, 14], [-2, 28], [41, 0], [6, -37], [34, -54], [-19, -27], [-2, -41], [-11, -18], [20, -42], [-6, 29], [-39, 8], [-40, -4], [-48, 41], [36, -24], [-24, 0], [31, 26], [6, 15], [-35, -10], [0, 30], [-7, -16], [-1, -16], [-49, 24], [46, -29], [-9, 11], [14, 16], [27, 15], [20, -42], [-40, 14], [39, -32], [37, 24], [-21, 7], [12, 12], [33, -16], [-6, 29], [54, 6], [-66, -4], [24, -31], [19, 22], [-36, 36], [50, -36], [5, -15], [66, -14], [18, 33], [7, -15], [-20, 56], [0, 38], [-29, -15], [25, -49], [-57, 24], [-11, 6], [-21, 26], [36, -45], [34, -42], [-1, -34], [-3, 32], [13, -1], [5, -24], [-49, 36], [44, -37], [19, 4], [-1, -10], [18, -43], [-27, -6], [23, -20], [-24, 0], [6, 47], [8, -42], [-31, 61], [-43, 40], [36, 22], [21, 0], [-2, 11], [-11, 48], [-22, 50], [13, -10], [63, -36], [-53, 24], [63, -4], [17, -31], [-10, -11], [36, -68], [-1, 29], [42, -36], [42, 0], [45, -18], [9, -50], [-56, 18], [13, 31], [22, -7], [64, 14], [34, 32], [-22, 4], [-13, -38], [68, -24], [37, -23], [52, -51], [43, -12], [-1, -6], [53, -32], [17, -25], [-52, 16], [-16, 10], [-7, -20], [-7, 26], [44, -42], [-45, 25], [2, 33], [-58, 50], [-10, 1], [21, 17], [44, -69], [-59, -4], [52, -39], [-14, 25], [-29, -8], [24, -52], [63, -22], [-57, 19], [39, -81], [-36, 62], [-15, 18], [39, -5], [6, 44], [25, 32], [58, -41], [-31, 45], [-17, 14], [16, 44], [36, -50], [13, 37], [-28, -31], [-54, -17], [37, -25], [66, -22], [-62, -5], [25, -10], [-8, -2], [50, -7], [-27, 13], [-3, -8], [25, -30], [-21, 32], [-59, 14], [-74, 35], [13, -8], [68, -16], [-1, 0], [-14, -46], [29, -6], [-46, 32], [-61, -2], [18, 1], [-41, -12], [-48, -26], [-48, -4], [17, -45], [42, -81], [-50, 30], [-47, -17], [76, 4], [85, 1], [10, 17], [44, -38], [2, 5], [-14, 35], [-11, -22], [-7, 12], [-2, -5], [24, -21], [-38, -20], [38, 7], [28, -32], [-29, -41], [-37, 45], [19, -62], [45, 17], [-15, 17], [-16, -21], [37, -18], [57, -50], [-27, -15], [-19, 5], [-57, 64], [18, 20], [-40, 78], [-8, 10], [-49, 51], [-60, 56], [37, -68], [-23, -18], [40, 11], [39, -2], [-28, 31], [-7, -34], [-34, -19], [-53, 46], [-62, 67], [56, -61], [-39, 64], [-38, 43], [38, -34], [-45, 82], [-5, 20], [32, 32], [73, 3], [53, -1], [11, 0], [-57, 75], [78, -9], [-33, -46], [29, -12], [-2, 10], [-21, 32], [0, 44], [41, 6]]
